"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../../util/index");
const CDEntry_1 = __importDefault(require("../headers/CDEntry"));
const EOCD_1 = __importDefault(require("../headers/EOCD"));
const EOCD64_1 = __importDefault(require("../headers/EOCD64"));
const EOCD64Locator_1 = __importDefault(require("../headers/EOCD64Locator"));
const LocalFile_1 = __importDefault(require("../headers/LocalFile"));
class Zip {
    constructor(pathOrUrl) {
        /** Zip size */
        this.size = -1;
        /** Central Directory Entries */
        this.files = new Map();
        this.path = pathOrUrl;
    }
    /**  */
    async init() {
        await this.preInit();
        await this.getEocd();
        await this.getCentralDir();
    }
    async get(entry, checkIntegrity = false) {
        if (entry.fileName.endsWith('/'))
            throw new Error('Downloading directories isnt supported');
        const headerSize = 30 + entry.fileNameLength + entry.extraFieldLength;
        const zip64Constant = BigInt(0xffffffff);
        // Use bigints to only deal with a single type
        let offset = BigInt(entry.offset);
        let size = BigInt(entry.compressedSize);
        let uncompressedSize = BigInt(entry.uncompressedSize);
        if (offset === zip64Constant
            && entry.extraField.zip64
            && entry.extraField.zip64.offset > BigInt(-1)) {
            offset = entry.extraField.zip64.offset;
        }
        if (size === zip64Constant
            && entry.extraField.zip64
            && entry.extraField.zip64.compressedSize > BigInt(-1)) {
            size = entry.extraField.zip64.compressedSize;
        }
        if (uncompressedSize === zip64Constant
            && entry.extraField.zip64
            && entry.extraField.zip64.uncompressedSize > BigInt(-1)) {
            uncompressedSize = entry.extraField.zip64.uncompressedSize;
        }
        // get extra 0xfff bytes to take account mismatch in extrafield lengths
        const data = await this.partialGet(offset, offset + size + BigInt(headerSize) + BigInt(0xfff));
        // 0xff more bytes because extrafield length mismatch
        const header = new LocalFile_1.default(data.slice(0, headerSize + 0xff));
        const fileData = data.slice(30 + header.extraFieldLength + header.fileNameLength, 
        // Use the compressed size from the CD entry so I dont need to deal
        // with the data descriptor stuff
        30 + header.extraFieldLength + header.fileNameLength + Number(size));
        if (BigInt(fileData.length) !== size)
            throw new Error('Invalid Compressed File Length');
        let uncompressedData;
        if (entry.compression === 8) {
            uncompressedData = await index_1.inflate(fileData);
        }
        else if (entry.compression === 0) {
            uncompressedData = fileData;
        }
        else {
            throw new Error('Unsupported compression method');
        }
        if (BigInt(uncompressedData.length) !== uncompressedSize) {
            throw new Error('Invalid Uncompressed File Length');
        }
        if (checkIntegrity) {
            // TODO: check Integrity before returning data;
        }
        return uncompressedData;
    }
    /** */
    async getEocd() {
        const bufSize = Math.min(22 + 0xff + 20, this.size);
        const data = await this.partialGet(this.size - bufSize, this.size);
        this.eocd = new EOCD_1.default(data);
        if (this.eocd && this.eocd.cdOffset === 0xFFFFFFFF) {
            this.eocd64Locator = new EOCD64Locator_1.default(data);
            const eocd64Buf = await this.partialGet(this.eocd64Locator.offset, this.size - 22 + 0xff + 20);
            this.eocd64 = new EOCD64_1.default(eocd64Buf);
        }
    }
    /** */
    async getCentralDir() {
        let offset = BigInt(this.eocd.cdOffset);
        let size = BigInt(this.eocd.cdSize);
        if (offset === BigInt(0xFFFFFFFF) && this.eocd64) {
            offset = this.eocd64.cdOffset;
        }
        if (size === BigInt(0xFFFFFFFF) && this.eocd64) {
            size = this.eocd64.cdSize;
        }
        const data = await this.partialGet(offset, offset + size);
        const signature = Buffer.alloc(4);
        signature.writeUInt32LE(CDEntry_1.default.signature, 0);
        const pieces = index_1.bufferSplit(data, signature);
        pieces.forEach((piece) => {
            const entry = new CDEntry_1.default(piece);
            this.files.set(entry.fileName, entry);
        });
    }
}
exports.default = Zip;
//# sourceMappingURL=Zip.js.map